/*
 * Copyright 2016 - 2025 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <ox/std/algorithm.hpp>

#include "deletetilescommand.hpp"

namespace nostalgia::gfx {

gfx::DeleteTilesCommand::DeleteTilesCommand(
		TileSheet &img,
		TileSheet::SubSheetIdx idx,
		std::size_t tileIdx,
		std::size_t tileCnt) noexcept:
		m_img(img),
		m_idx(std::move(idx)) {
	const unsigned bytesPerTile = m_img.bpp == 4 ? PixelsPerTile / 2 : PixelsPerTile;
	m_deletePos = tileIdx * bytesPerTile;
	m_deleteSz = tileCnt * bytesPerTile;
	m_deletedPixels.resize(m_deleteSz);
	// copy pixels to be erased
	{
		auto &s = getSubSheet(m_img, m_idx);
		auto dst = m_deletedPixels.begin();
		auto src = s.pixels.begin() + m_deletePos;
		ox::copy_n(src, m_deleteSz, dst);
	}
}

ox::Error gfx::DeleteTilesCommand::redo() noexcept {
	auto &s = getSubSheet(m_img, m_idx);
	auto &p = s.pixels;
	auto srcPos = m_deletePos + m_deleteSz;
	auto const src = &p[srcPos];
	auto const dst1 = &p[m_deletePos];
	auto const dst2 = &p[(p.size() - m_deleteSz)];
	ox::memmove(dst1, src, p.size() - srcPos);
	ox::memset(dst2, 0, m_deleteSz * sizeof(decltype(p[0])));
	return {};
}

ox::Error DeleteTilesCommand::undo() noexcept {
	auto &s = getSubSheet(m_img, m_idx);
	auto &p = s.pixels;
	auto const src = &p[m_deletePos];
	auto const dst1 = &p[m_deletePos + m_deleteSz];
	auto const dst2 = src;
	auto const sz = p.size() - m_deletePos - m_deleteSz;
	ox::memmove(dst1, src, sz);
	ox::memcpy(dst2, m_deletedPixels.data(), m_deletedPixels.size());
	return {};
}

int DeleteTilesCommand::commandId() const noexcept {
	return static_cast<int>(CommandId::DeleteTile);
}

TileSheet::SubSheetIdx const&DeleteTilesCommand::subsheetIdx() const noexcept {
	return m_idx;
}

}
