/*
 * Copyright 2016 - 2025 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#pragma once

#include <functional>

#include <ox/std/string.hpp>
#include <ox/std/vector.hpp>

#include <turbine/context.hpp>

#include <studio/itemmaker.hpp>

namespace studio {

class ItemMaker;

struct EditorMaker {
	using Func = std::function<ox::Result<class BaseEditor*>(ox::StringViewCR)>;
	ox::Vector<ox::String> fileTypes;
	Func make;
};

class Module {
	public:
		virtual ~Module() noexcept = default;

		virtual ox::Vector<EditorMaker> editors(studio::StudioContext &ctx) const;

		virtual ox::Vector<ox::UPtr<ItemMaker>> itemMakers(studio::StudioContext&) const;

		virtual ox::Vector<ox::UPtr<ItemTemplate>> itemTemplates(studio::StudioContext&) const;

};

template<typename Editor>
[[nodiscard]]
EditorMaker editorMaker(StudioContext &ctx, ox::StringParam ext) noexcept {
	return {
			{std::move(ext)},
			[&ctx](ox::StringViewCR path) -> ox::Result<BaseEditor*> {
				return ox::makeCatch<Editor>(ctx, path);
			}
	};
}

template<typename Editor>
[[nodiscard]]
EditorMaker editorMaker(StudioContext &ctx, std::initializer_list<ox::StringView> exts) noexcept {
	return {
			[&exts] {
				ox::Vector<ox::String> fileTypes;
				fileTypes.reserve(exts.size());
				for (auto &s : exts) {
					fileTypes.emplace_back(s);
				}
				return fileTypes;
			}(),
			[&ctx](ox::StringViewCR path) -> ox::Result<BaseEditor*> {
				return ox::makeCatch<Editor>(ctx, path);
			}
	};
}

}
